/*
 * Copyright 2012 Canonical Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "friendsdispatcher.h"
#include <glib-object.h>

FriendsDispatcherQml::FriendsDispatcherQml(QObject *parent) :
    QObject(parent),
    m_dispatcher(0)
{
    GError* error = NULL;
    m_dispatcher = friends_dispatcher_new(&error);
    if (error != NULL) 
        g_error_free (error);
}

FriendsDispatcherQml::~FriendsDispatcherQml()
{
    g_object_unref(m_dispatcher);
}

QStringList FriendsDispatcherQml::featuresForProtocol(const QString& protocol) const
{
    QStringList features;
    int length;
    QByteArray byte_array = protocol.toUtf8();
    GError* error = NULL;

    gchar** c_features = friends_dispatcher_features(m_dispatcher, byte_array.data(), &length, &error);
    for (int i = 0; i < length; i++) {
        features.append(QString::fromUtf8(c_features[i]));
    }
    if (error != NULL) 
        g_error_free (error);
    return features;
}

void FriendsDispatcherQml::uploadForAccount(const int& account_id, const QString& uri, const QString& description) const
{
    QByteArray path = uri.toUtf8();
    QByteArray desc = description.toUtf8();
    GError* error = NULL;
    friends_dispatcher_upload(m_dispatcher, account_id, path.data(), desc.data(), &error);

    if (error != NULL) 
        g_error_free (error);
}

void FriendsDispatcherQml::sendForAccount(const int& account_id, const QString& message) const
{
    QByteArray msg = message.toUtf8();
    uint acct = (uint)account_id;
    GError* error = NULL;
    friends_dispatcher_send_message(m_dispatcher, &acct, msg.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::send(const QString& message) const
{
    QByteArray msg = message.toUtf8();
    GError* error = NULL;
    friends_dispatcher_send_message(m_dispatcher, NULL, msg.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::sendReply(const int& account_id, const QString& message_id, const QString& message) const
{
    QByteArray msg = message.toUtf8();
    QByteArray msg_id = message_id.toUtf8();
    GError* error = NULL;
    friends_dispatcher_send_reply(m_dispatcher, account_id, msg_id.data(), msg.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::retweetCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_retweet_finish(dispatcher->m_dispatcher, res, &result, &error);
        QString errorMessage;
        if (result == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->retweetComplete(QString::fromUtf8(result), success, errorMessage);
        if (error != NULL)
            g_error_free (error);
    }
}

void FriendsDispatcherQml::retweetAsync(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    friends_dispatcher_retweet_async(m_dispatcher, account_id, msg_id.data(),
                                 &unlikeCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::retweet(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    GError* error = NULL;
    friends_dispatcher_retweet(m_dispatcher, account_id, msg_id.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::likeCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_like_finish(dispatcher->m_dispatcher, res, &result, &error);
        QString errorMessage;
        if (result == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->likeComplete(QString::fromUtf8(result), success, errorMessage);
        if (error != NULL)
            g_error_free (error);
    }
}

void FriendsDispatcherQml::likeAsync(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    friends_dispatcher_like_async(m_dispatcher, account_id, msg_id.data(),
                                 &unlikeCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::like(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    GError* error = NULL;
    friends_dispatcher_like(m_dispatcher, account_id, msg_id.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::unlikeCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_unlike_finish(dispatcher->m_dispatcher, res, &result, &error);
        QString errorMessage;
        if (result == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->unlikeComplete(QString::fromUtf8(result), success, errorMessage);
        if (error != NULL)
            g_error_free (error);
    }
}

void FriendsDispatcherQml::unlikeAsync(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    friends_dispatcher_unlike_async(m_dispatcher, account_id, msg_id.data(),
                                 &unlikeCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::unlike(const int& account_id, const QString& message_id) const
{
    QByteArray msg_id = message_id.toUtf8();
    GError* error = NULL;
    friends_dispatcher_unlike(m_dispatcher, account_id, msg_id.data(), &error);

    if (error != NULL)
        g_error_free (error);
}

void FriendsDispatcherQml::uploadCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result_url;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_upload_finish(dispatcher->m_dispatcher, res, &result_url, &error);
        QString errorMessage;
        if (result_url == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->uploadComplete(QString::fromUtf8(result_url), success, errorMessage);
        if (error != NULL) 
            g_error_free (error);
    }
}

void FriendsDispatcherQml::uploadForAccountAsync(const int& account_id, const QString& uri, const QString& description) const
{
    QByteArray path = uri.toUtf8();
    QByteArray desc = description.toUtf8();
    friends_dispatcher_upload_async(m_dispatcher, account_id, path.data(), desc.data(),
                                 &uploadCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::sendCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_send_message_finish(dispatcher->m_dispatcher, res, &result, &error);
        QString errorMessage;
        if (result == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->sendComplete(QString::fromUtf8(result), success, errorMessage);
        if (error != NULL) 
            g_error_free (error);
    }
}

void FriendsDispatcherQml::sendAsync(const QString& message) const
{
    QByteArray msg = message.toUtf8();
    friends_dispatcher_send_message_async(m_dispatcher, NULL, msg.data(),
                                 &sendCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::sendForAccountAsync(const int& account_id, const QString& message) const
{
    QByteArray msg = message.toUtf8();
    uint acct = (uint)account_id;
    friends_dispatcher_send_message_async(m_dispatcher, &acct, msg.data(),
                                 &sendCompleteCallback, (gpointer) this);
}

void FriendsDispatcherQml::sendReplyCompleteCallback(GObject *source_object, GAsyncResult *res, gpointer user_data)
{
    Q_UNUSED(source_object);

    FriendsDispatcherQml* dispatcher = (FriendsDispatcherQml*) user_data;
    if (dispatcher) {
        gchar* result;
        GError* error = NULL;
        bool success = TRUE;
        friends_dispatcher_send_reply_finish(dispatcher->m_dispatcher, res, &result, &error);
        QString errorMessage;
        if (result == NULL || error != NULL) {
            success = FALSE;
            errorMessage = QString::fromUtf8(error->message);
        }
        Q_EMIT dispatcher->sendComplete(QString::fromUtf8(result), success, errorMessage);
        if (error != NULL)
            g_error_free (error);
    }
}

void FriendsDispatcherQml::sendReplyAsync(const int& account_id, const QString& message_id, const QString& message) const
{
    QByteArray msg = message.toUtf8();
    QByteArray msg_id = message_id.toUtf8();
    friends_dispatcher_send_reply_async(m_dispatcher, account_id, msg_id.data(), msg.data(),
                                 &sendReplyCompleteCallback, (gpointer) this);
}

